## Overview

This charm is a component of the Apache Hadoop platform.  It is intended
to be deployed with the other components using the bundle:
`bundle:~bigdata-charmers/apache-hadoop`

**What is Apache Hadoop?**

The Apache Hadoop software library is a framework that allows for the
distributed processing of large data sets across clusters of computers
using a simple programming model.

It is designed to scale up from single servers to thousands of machines,
each offering local computation and storage. Rather than rely on hardware
to deliver high-avaiability, the library itself is designed to detect
and handle failures at the application layer, so delivering a
highly-availabile service on top of a cluster of computers, each of
which may be prone to failures.

Apache Hadoop 2.4.1 consists of significant improvements over the previous stable
release (hadoop-1.x).

Here is a short overview of the improvments to both HDFS and MapReduce.

 - **HDFS Federation**
   In order to scale the name service horizontally, federation uses multiple
   independent Namenodes/Namespaces. The Namenodes are federated, that is, the
   Namenodes are independent   and don't require coordination with each other.
   The datanodes are used as common storage for blocks by all the Namenodes.
   Each datanode registers with all the Namenodes in the cluster.   Datanodes
   send periodic heartbeats and block reports and handles commands from the
   Namenodes.

   More details are available in the HDFS Federation document:
   <http://hadoop.apache.org/docs/r2.4.1/hadoop-project-dist/hadoop-hdfs/Federation.html>

 - **MapReduce NextGen aka YARN aka MRv2**
   The new architecture introduced in hadoop-0.23, divides the two major functions of the
   JobTracker: resource management and job life-cycle management into separate components.
   The new ResourceManager manages the global assignment of compute resources to
   applications and the per-application ApplicationMaster manages the application‚
   scheduling and coordination.
   An application is either a single job in the sense of classic MapReduce jobs or a DAG of
   such jobs.

   The ResourceManager and per-machine NodeManager daemon, which manages the user
   processes on   that machine, form the computation fabric.

   The per-application ApplicationMaster is, in effect, a framework specific
   library and is tasked with negotiating resources from the ResourceManager and
   working with the NodeManager(s) to execute and monitor the tasks.

   More details are available in the YARN document:
   <http://hadoop.apache.org/docs/r2.4.1/hadoop-yarn/hadoop-yarn-site/YARN.html>

## Usage

This charm manages the hadoop client node.  It is intended to be used with
`apache-hadoop-hdfs-master` and `apache-hadoop-yarn-master`.

### Simple Usage: Single YARN / HDFS master deployment

In this configuration, the YARN and HDFS master components run on the same
machine.  This is useful for lower-resource deployments::

    juju deploy apache-hadoop-hdfs-master hdfs-master
    juju deploy apache-hadoop-hdfs-checkpoint secondary-namenode --to 1
    juju deploy apache-hadoop-yarn-master yarn-master --to 1
    juju deploy apache-hadoop-compute-slave compute-slave
    juju deploy apache-hadoop-client client
    juju add-relation yarn-master hdfs-master
    juju add-relation secondary-namenode hdfs-master
    juju add-relation compute-slave yarn-master
    juju add-relation compute-slave hdfs-master
    juju add-relation client yarn-master
    juju add-relation client hdfs-master

Note that the machine number (`--to 1`) should match the machine number
for the `hdfs-master` charm.  If you previously deployed other services
in your environment, you may need to adjust the machine number appropriately.


### Scale Out Usage: Separate HDFS, YARN, and compute nodes

In this configuration the HDFS and YARN deployments operate on
different service units as separate services::

    juju deploy apache-hadoop-hdfs-master hdfs-master
    juju deploy apache-hadoop-hdfs-checkpoint secondary-namenode
    juju deploy apache-hadoop-yarn-master yarn-master
    juju deploy apache-hadoop-compute-slave compute-slave -n 3
    juju deploy apache-hadoop-client client
    juju add-relation yarn-master hdfs-master
    juju add-relation secondary-namenode hdfs-master
    juju add-relation compute-slave yarn-master
    juju add-relation compute-slave hdfs-master
    juju add-relation client yarn-master
    juju add-relation client hdfs-master

The `-n 3` option can be adjusted according to the number of compute nodes
you need.  You can also add additional compute nodes later::

    juju add-unit compute-slave -n 2


### To deploy a Hadoop service with elasticsearch service::
    # deploy ElasticSearch locally:
    **juju deploy elasticsearch elasticsearch**
    # elasticsearch-hadoop.jar file will be added to LIBJARS path
    # Recommanded to use hadoop -libjars option to included elk jar file
    **juju add-unit -n elasticsearch**
    # deploy hive service by any senarios mentioned above
    # associate Hive with elasticsearch
    **juju add-relation {hadoop master}:elasticsearch elasticsearch:client**


## Deploying in Network-Restricted Environments

The Apache Hadoop charms can be deployed in environments with limited network
access. To deploy in this environment, you will need a local mirror to serve
the packages and resources required by these charms.

### Mirroring Packages

You can setup a local mirror for apt packages using squid-deb-proxy.
For instructions on configuring juju to use this, see the
[Juju Proxy Documentation](https://juju.ubuntu.com/docs/howto-proxies.html).

### Mirroring Resources

In addition to apt packages, the Apache Hadoop charms require a few binary
resources, which are normally hosted on Launchpad. If access to Launchpad
is not available, the `jujuresources` library makes it easy to create a mirror
of these resources:

    sudo pip install jujuresources
    juju resources fetch --all apache-hadoop-client/resources.yaml -d /tmp/resources
    juju resources serve -d /tmp/resources

This will fetch all of the resources needed by this charm and serve them via a
simple HTTP server. You can then set the `resources_mirror` config option to
have the charm use this server for retrieving resources.

You can fetch the resources for all of the Apache Hadoop charms
(`apache-hadoop-hdfs-master`, `apache-hadoop-yarn-master`,
`apache-hadoop-compute-slave`, `apache-hadoop-client`, etc) into a single
directory and serve them all with a single `juju resources serve` instance.


## Contact Information
amir sanjar <amir.sanjar@canonical.com>

## Hadoop
- [Apache Hadoop](http://hadoop.apache.org/) home page
- [Apache Hadoop bug trackers](http://hadoop.apache.org/issue_tracking.html)
- [Apache Hadoop mailing lists](http://hadoop.apache.org/mailing_lists.html)
- [Apache Hadoop Juju Charm](http://jujucharms.com/?text=hadoop)
