#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import jujuresources


def bootstrap_resources():
    """
    Install required resources defined in resources.yaml
    """
    mirror_url = jujuresources.config_get('resources_mirror')
    if not jujuresources.fetch(mirror_url=mirror_url):
        jujuresources.juju_log('Resources unavailable; manual intervention required', 'ERROR')
        return False
    jujuresources.install(['pathlib', 'pyaml', 'six', 'charmhelpers'])
    return True


def manage():
    if not bootstrap_resources():
        # defer until resources are available, since charmhelpers, and thus
        # the framework, (will be) managed by jujuresources
        return

    from charmhelpers.core import ch_framework
    from charmhelpers.contrib import bigdata

    hadoop = bigdata.handlers.apache.HadoopBase()
    hdfs = bigdata.handlers.apache.HDFS(hadoop)
    yarn = bigdata.handlers.apache.YARN(hadoop)
    manager = ch_framework.Manager([
        {
            'name': 'hadoop-base',
            'requires': [
                hadoop.verify_conditional_resources,
            ],
            'callbacks': [
                hadoop.install,
            ],
        },
        {
            'name': 'client',
            'provides': [
                bigdata.relations.HadoopClient(hadoop),
            ],
            'requires': [
                hadoop.is_installed,
                bigdata.relations.HDFSMaster(spec=hadoop.spec),
                bigdata.relations.YARNMaster(spec=hadoop.spec),
            ],
            'callbacks': [
                hdfs.configure_client,
                yarn.configure_client,
            ],
            'cleanup': [],
        },
    ])
    manager.manage()


if __name__ == '__main__':
    manage()
